@@ -160,6 +160,36 @@ class KTTSD : public QObject, virtual public KSpeech
         */
         virtual uint setText(const QString &text, const QString &talker=NULL);
 
+        /**
+        * Say a plain text job.  This is a convenience method that
+        * combines @ref setText and @ref startText into a single call.
+        * @param text           The message to be spoken.
+        * @param talker         Code for the talker to do the speaking.  Example "en".
+        *                       If NULL, defaults to the user's default plugin.
+        *                       If no plugin has been configured for the specified Talker code,
+        *                       defaults to the closest matching talker.
+        * @return               Job number.
+        *
+        * Plain text is parsed into individual sentences using the current sentence delimiter.
+        * Call @ref setSentenceDelimiter to change the sentence delimiter prior to
+        * calling setText.
+        * Call @ref getTextCount to retrieve the sentence count after calling setText.
+        *
+        * The text may contain speech mark language, such as Sable, JSML, or SSML,
+        * provided that the speech plugin/engine support it.  In this case,
+        * sentence parsing follows the semantics of the markup language.
+        *
+        * The job is marked speakable.
+        * If there are other speakable jobs preceeding this one in the queue,
+        * those jobs continue speaking and when finished, this job will begin speaking.
+        * If there are no other speakable jobs preceeding this one, it begins speaking.
+        *
+        * @see getTextCount
+        *
+        * @since KDE 3.5
+        */
+        virtual uint sayText(const QString &text, const QString &talker);
+
         /**
         * Adds another part to a text job.  Does not start speaking the text.
         * (thread safe)
@@ -390,7 +420,11 @@ class KTTSD : public QObject, virtual public KSpeech
         *
         * The job is marked speakable.
         *
-        * If the job was not paused, it is the same as calling @ref startText.
+        * If the job is currently speaking, or is waiting to be spoken (speakable 
+        * state), the resumeText() call is ignored.
+        *
+        * If the job is currently queued, or is finished, it is the same as calling
+        * @ref startText .
         *
         * If there are speaking jobs preceeding this one in the queue, those jobs continue speaking and,
         * when finished this job will begin speaking where it left off.
@@ -609,6 +643,7 @@ class kspeech : public QObject, virtual public KSpeech
         virtual ASYNC sayMessage(const QString &message, const QString &talker);
         virtual ASYNC setSentenceDelimiter(const QString &delimiter);
         virtual uint setText(const QString &text, const QString &talker);
+        virtual uint sayText(const QString &text, const QString &talker);
         virtual int appendText(const QString &text, uint jobNum=0);
         virtual uint setFile(const QString &filename, const QString &talker,
                              const QString& encoding);
