@@ -124,14 +124,29 @@ Speaker::Speaker( SpeechData*speechData, TalkerMgr* talkerMgr,
     m_playerOption = m_speechData->config->readNumEntry("AudioOutputMethod", 0);  // default to aRts.
     // Map 50% to 100% onto 2.0 to 0.5.
     m_audioStretchFactor = 1.0/(float(m_speechData->config->readNumEntry("AudioStretchFactor", 100))/100.0);
-    m_speechData->config->setGroup("GStreamerPlayer");
-    m_gstreamerSinkName = m_speechData->config->readEntry("SinkName", "osssink");
+    switch (m_playerOption)
+    {
+        case 0: break;
+        case 1:
+            m_speechData->config->setGroup("GStreamerPlayer");
+            m_sinkName = m_speechData->config->readEntry("SinkName", "osssink");
+            break;
+        case 2:
+            m_speechData->config->setGroup("ALSAPlayer");
+            m_sinkName = m_speechData->config->readEntry("PcmName", "default");
+            break;
+        case 3:
+            m_speechData->config->setGroup("aKodePlayer");
+            m_sinkName = m_speechData->config->readEntry("SinkName", "auto");
+            break;
+    }
     // Connect timer timeout signal.
     connect(m_timer, SIGNAL(timeout()), this, SLOT(slotTimeout()));
 
     // Connect plugins to slots.
     QPtrList<PlugInProc> plugins = m_talkerMgr->getLoadedPlugIns();
-    for (uint ndx = 0; ndx < plugins.count(); ndx++)
+    const int pluginsCount = plugins.count();
+    for (int ndx = 0; ndx < pluginsCount; ++ndx)
     {
         PlugInProc* speech = plugins.at(ndx);
         connect(speech, SIGNAL(synthFinished()),
@@ -227,6 +242,7 @@ void Speaker::doUtterances()
             }
             // Loop through utterance queue.
             int waitingCnt = 0;
+            int waitingMsgCnt = 0;
             int transformingCnt = 0;
             bool playing = false;
             int synthingCnt = 0;
@@ -235,6 +251,7 @@ void Speaker::doUtterances()
             for (it = itBegin; it != itEnd; ++it)
             {
                 uttState utState = it->state;
+                uttType utType = it->utType;
                 switch (utState)
                 {
                     case usNone:
@@ -283,7 +300,7 @@ void Speaker::doUtterances()
                         // If first in queue, emit signal.
                         if (it == itBegin)
                         {
-                            if (it->utType == utStartOfJob)
+                            if (utType == utStartOfJob)
                             {
                                 m_speechData->setTextJobState(
                                     it->sentence->jobNum, KSpeech::jsSpeaking);
@@ -360,8 +377,14 @@ void Speaker::doUtterances()
                             {
                                 playing = true;
                                 m_again = true;
-                            } else ++waitingCnt;
-                        } else ++waitingCnt;
+                            } else {
+                                ++waitingCnt;
+                                if (utType == utWarning || utType == utMessage) ++waitingMsgCnt;
+                            }
+                        } else {
+                            ++waitingCnt;
+                            if (utType == utWarning || utType == utMessage) ++waitingMsgCnt;
+                        }
                         break;
                     }
                     case usPlaying:
@@ -378,8 +401,14 @@ void Speaker::doUtterances()
                             {
                                 playing = true;
                                 m_again = true;
-                            } else ++waitingCnt;
-                        } else ++waitingCnt;
+                            } else {
+                                ++waitingCnt;
+                                if (utType == utWarning || utType == utMessage) ++waitingMsgCnt;
+                            }
+                        } else {
+                            ++waitingCnt;
+                            if (utType == utWarning || utType == utMessage) ++waitingMsgCnt;
+                        }
                         break;
                     }
                     case usWaitingSay:
@@ -405,9 +434,18 @@ void Speaker::doUtterances()
                                     playing = true;
                                     it->plugin->sayText(it->sentence->text);
                                     m_again = true;
-                                } else ++waitingCnt;
-                            } else ++waitingCnt;
-                        } else ++waitingCnt;
+                                } else {
+                                    ++waitingCnt;
+                                    if (utType == utWarning || utType == utMessage) ++waitingMsgCnt;
+                                }
+                            } else {
+                                ++waitingCnt;
+                                if (utType == utWarning || utType == utMessage) ++waitingMsgCnt;
+                            }
+                        } else {
+                            ++waitingCnt;
+                            if (utType == utWarning || utType == utMessage) ++waitingMsgCnt;
+                        }
                         break;
                     }
                     case usWaitingSynth:
@@ -426,6 +464,7 @@ void Speaker::doUtterances()
                             m_again = true;
                         }
                         ++waitingCnt;
+                        if (utType == utWarning || utType == utMessage) ++waitingMsgCnt;
                         break;
                     }
                     case usSaying:
@@ -439,6 +478,7 @@ void Speaker::doUtterances()
                         } else {
                             playing = true;
                             ++waitingCnt;
+                            if (utType == utWarning || utType == utMessage) ++waitingMsgCnt;
                         }
                         break;
                     }
@@ -454,11 +494,22 @@ void Speaker::doUtterances()
                             m_again = true;
                         } else ++synthingCnt;
                         ++waitingCnt;
+                        if (utType == utWarning || utType == utMessage) ++waitingMsgCnt;
                         break;
                     }
                     case usFinished: break;
                 }
             }
+            // See if there are any messages or warnings to process.
+            // We keep up to 2 such utterances in the queue.
+            if ((waitingMsgCnt < 2) && (transformingCnt < 3))
+            {
+                if (m_speechData->warningInQueue() || m_speechData->messageInQueue())
+                {
+                    getNextUtterance();
+                    m_again = true;
+                }
+            }
             // Try to keep at least two utterances in the queue waiting to be played,
             // and no more than 3 transforming at one time.
             if ((waitingCnt < 2) && (transformingCnt < 3))
@@ -586,7 +637,11 @@ void Speaker::pauseText(const uint jobNum)
  *
  * The job is marked speakable.
  *
- * If the job was not paused, it is the same as calling @ref startText.
+ * If the job is currently speaking, or is waiting to be spoken (speakable 
+ * state), the resumeText() call is ignored.
+ *
+ * If the job is currently queued, or is finished, it is the same as calling
+ * @ref startText .
  *
  * If there are speaking jobs preceeding this one in the queue, those jobs continue speaking and,
  * when finished this job will begin speaking where it left off.
@@ -596,15 +651,24 @@ void Speaker::pauseText(const uint jobNum)
  */
 void Speaker::resumeText(const uint jobNum)
 {
-    if (m_speechData->getTextJobState(jobNum) != KSpeech::jsPaused)
-        startText(jobNum);
-    else
+    int state = m_speechData->getTextJobState(jobNum);
+    switch (state)
     {
-        if (jobNum == m_currentJobNum)
-            m_speechData->setTextJobState(jobNum, KSpeech::jsSpeaking);
-        else
-            m_speechData->setTextJobState(jobNum, KSpeech::jsSpeakable);
-        doUtterances();
+        case KSpeech::jsQueued:
+        case KSpeech::jsFinished:
+            startText(jobNum);
+            break;
+        case KSpeech::jsSpeakable:
+        case KSpeech::jsSpeaking:
+            doUtterances();
+            break;
+        case KSpeech::jsPaused:
+            if (jobNum == m_currentJobNum)
+                m_speechData->setTextJobState(jobNum, KSpeech::jsSpeaking);
+            else
+                m_speechData->setTextJobState(jobNum, KSpeech::jsSpeakable);
+            doUtterances();
+            break;
     }
 }
 
@@ -993,7 +1057,7 @@ bool Speaker::getNextUtterance()
                         (it->state == usSaying))) ++it;
                 // If now pointing at a text message, we are interrupting.
                 // Insert optional Interruption message and sound.
-                if (it != itEnd) interrupting = (it->utType == utText);
+                if (it != itEnd) interrupting = (it->utType == utText && it->state != usPaused);
                 if (interrupting)
                 {
                     if (m_speechData->textPreSndEnabled)
@@ -1019,7 +1083,8 @@ bool Speaker::getNextUtterance()
                         Utt intrUtt;
                         intrUtt.sentence = new mlText;
                         intrUtt.sentence->text = m_speechData->textPreMsg;
-                        intrUtt.sentence->talker = utt->sentence->talker;
+                        // Interruptions are spoken using default Talker.
+                        intrUtt.sentence->talker = QString::null;
                         intrUtt.sentence->appId = utt->sentence->appId;
                         intrUtt.sentence->jobNum = utt->sentence->jobNum;
                         intrUtt.sentence->seq = 0;
@@ -1037,7 +1102,7 @@ bool Speaker::getNextUtterance()
             }
             // Insert the new message or warning.
             it = m_uttQueue.insert(it, *utt);
-            it++;
+            ++it;
             // Resumption message and sound.
             if (interrupting)
             {
@@ -1064,7 +1129,7 @@ bool Speaker::getNextUtterance()
                     Utt resUtt;
                     resUtt.sentence = new mlText;
                     resUtt.sentence->text = m_speechData->textPostMsg;
-                    resUtt.sentence->talker = utt->sentence->talker;
+                    resUtt.sentence->talker = QString::null;
                     resUtt.sentence->appId = utt->sentence->appId;
                     resUtt.sentence->jobNum = utt->sentence->jobNum;
                     resUtt.sentence->seq = 0;
@@ -1218,7 +1283,7 @@ uttIterator Speaker::deleteUtterance(uttIterator it)
         case usPaused:
         case usPreempted:
         {
-                        // Note: Must call stop(), even if player not currently playing.  Why?
+            // Note: Must call stop(), even if player not currently playing.  Why?
             it->audioPlayer->stop();
             delete it->audioPlayer;
             break;
@@ -1231,8 +1296,12 @@ uttIterator Speaker::deleteUtterance(uttIterator it)
         {
             if (m_speechData->keepAudio)
             {
-                QString dest = m_speechData->keepAudioPath + "kttsd-" +
-                    QString("%1-%2").arg(it->sentence->jobNum).arg(it->sentence->seq) + ".wav";
+                QCString seqStr;
+                seqStr.sprintf("%08i", it->sentence->seq);    // Zero-fill to 8 chars.
+                QCString jobStr;
+                jobStr.sprintf("%08i", it->sentence->jobNum);
+                QString dest = m_speechData->keepAudioPath + "/kttsd-" +
+                    QString("%1-%2").arg(jobStr).arg(seqStr) + ".wav";
                 QFile::remove(dest);
                 QDir d;
                 d.rename(it->audioUrl, dest);
@@ -1450,6 +1519,16 @@ Player* Speaker::createPlayerObject()
                 plugInName = "kttsd_gstplugin";
                 break;
             }
+        case 2 :
+            {
+                plugInName = "kttsd_alsaplugin";
+                break;
+            }
+        case 3 :
+            {
+                plugInName = "kttsd_akodeplugin";
+                break;
+            }
         default:
             {
                 plugInName = "kttsd_artsplugin";
@@ -1461,7 +1540,7 @@ Player* Speaker::createPlayerObject()
 
     if(offers.count() == 1)
     {
-        // kdDebug() << "Speaker::createPlayerObject: Loading " << offers[0]->library() << endl;
+        kdDebug() << "Speaker::createPlayerObject: Loading " << offers[0]->library() << endl;
         KLibFactory *factory = KLibLoader::self()->factory(offers[0]->library().latin1());
         if (factory)
             player = 
@@ -1470,8 +1549,8 @@ Player* Speaker::createPlayerObject()
     }
     if (player == 0)
     {
-        // If we tried for GStreamer plugin and failed, fall back to aRts plugin.
-        if (m_playerOption == 1)
+        // If we tried for GStreamer or ALSA plugin and failed, fall back to aRts plugin.
+        if (m_playerOption != 0)
         {
             kdDebug() << "Speaker::createPlayerObject: Could not load " + plugInName + 
                 " plugin.  Falling back to aRts." << endl;
@@ -1491,8 +1570,7 @@ Player* Speaker::createPlayerObject()
                 return createPlayerObject();
             }
         }
-        else
-            player->setSinkName(m_gstreamerSinkName);
+    if (player) player->setSinkName(m_sinkName);
     return player;
 }
 
