@@ -215,7 +215,11 @@ class Speaker : public QObject{
         *
         * The job is marked speakable.
         *
-        * If the job was not paused, it is the same as calling @ref startText.
+        * If the job is currently speaking, or is waiting to be spoken (speakable 
+        * state), the resumeText() call is ignored.
+        *
+        * If the job is currently queued, or is finished, it is the same as calling
+        * @ref startText .
         *
         * If there are speaking jobs preceeding this one in the queue, those jobs continue speaking and,
         * when finished this job will begin speaking where it left off.
@@ -552,6 +556,7 @@ class Speaker : public QObject{
         * Which audio player to use.
         *  0 = aRts
         *  1 = gstreamer
+        *  2 = ALSA
         */
         int m_playerOption;
 
@@ -561,9 +566,9 @@ class Speaker : public QObject{
         float m_audioStretchFactor;
 
         /**
-        * GStreamer sink name to use.
+        * GStreamer sink name to use, or ALSA PCM device name.
         */
-        QString m_gstreamerSinkName;
+        QString m_sinkName;
 
         /**
         * Timer for monitoring audio player.
