@@ -79,6 +79,26 @@ struct PooledFilterMgr {
     TalkerCode* talkerCode;     /* TalkerCode object passed to FilterMgr. */
 };
 
+/**
+ * Struct used to keep notification options.
+ */
+struct NotifyOptions {
+    QString eventName;
+    int action;
+    QString talker;
+    QString customMsg;
+};
+
+/**
+ * A list of notification options for a single app, indexed by event.
+ */
+typedef QMap<QString, NotifyOptions> NotifyEventMap;
+
+/**
+ * A list of notification event maps for all apps, indexed by app.
+ */
+typedef QMap<QString, NotifyEventMap> NotifyAppMap;
+
 /**
  * SpeechData class which is in charge of maintaining all the data on the memory.
  * It maintains queues and has methods to enque
@@ -526,25 +546,34 @@ class SpeechData : public QObject {
         QString keepAudioPath;
 
         /**
-        * Notify.
+        * Notification settings.
         */
         bool notify;
+        bool notifyExcludeEventsWithSound;
+        NotifyAppMap notifyAppMap;
+        int notifyDefaultPresent;
+        NotifyOptions notifyDefaultOptions;
 
         /**
-        * Notify popups only.
+        * Automatically start KTTSMgr whenever speaking.
         */
-        bool notifyPopupsOnly;
+        bool autoStartManager;
 
         /**
-        * Notify passive popups only.
+        * Automatically exit auto-started KTTSMgr when speaking finishes.
         */
-        bool notifyPassivePopupsOnly;
+        bool autoExitManager;
 
         /**
         * Configuration
         */
         KConfig *config;
 
+        /**
+        * True if at least one XML Transformer plugin for html is enabled.
+        */
+        bool supportsHTML;
+
     signals:
         /**
         * This signal is emitted whenever a new text job is added to the queue.
@@ -579,7 +608,7 @@ class SpeechData : public QObject {
         /**
         * Queue of warnings
         */
-        QPtrQueue<mlText> warnings;
+        QPtrQueue<mlJob> warnings;
 
         /**
         * Queue of messages
@@ -640,7 +669,7 @@ class SpeechData : public QObject {
         * If appId is NULL, returns the last job in the queue.
         * Does not change textJobs.current().
         */
-        mlJob* SpeechData::findAJobByAppId(const QCString& appId);
+        mlJob* findAJobByAppId(const QCString& appId);
 
         /**
         * Given a job and a sequence number, returns the part that sentence is in.
@@ -660,7 +689,7 @@ class SpeechData : public QObject {
         * @return               List of parsed sentences.
         */
 
-        QStringList SpeechData::parseText(const QString &text, const QCString &appId);
+        QStringList parseText(const QString &text, const QCString &appId);
 
         /**
         * Delete expired jobs.  At most, one finished job is kept on the queue.
@@ -688,6 +717,11 @@ class SpeechData : public QObject {
         */
         void doFiltering();
 
+        /**
+        * Loads notify events from a file.  Clearing data if clear is True.
+        */
+        void loadNotifyEventsFromFile( const QString& filename, bool clear);
+
     private slots:
         void slotFilterMgrFinished();
         void slotFilterMgrStopped();
